%option noyywrap
%{
#define YY_NO_UNPUT

#include "function.h"
#include "parser.tab.h"
#include <stdlib.h>
#include <string.h>

int yyparse(void);
%}

DIGITS [0-9]+
%%

 /* Always skip whitespace */
[ \t]*

 /* Power operator */
"**" { return POWER; }
"^" { return POWER; }

 /* Read numbers of the form with at least the decimal point and trailing
 * digits, such as .32, -234.45, .0, etc.  Numbers are only read in the BEGIN
 * state.
 */
{DIGITS}*(\.{DIGITS}+)(e[-+]?{DIGITS}+)? {
    return NUM;
}

 /* Read numbers with at least the integral part, such as +4234, -34e8, etc.
 * Numbers are only read in the BEGIN state.
 */
{DIGITS}+(\.{DIGITS}*)?(e[-+]?{DIGITS}+)? {
    return NUM;
}

[nN][aA][nN] { return NUM; }
[iI][nN][fF] { return NUM; }

 /* This detects textual input, and if it isn't pre-declared by the parser (in
 * other words, if it isn't a function), then it is returned as an identifier.
 */
[a-zA-Z_][a-zA-Z_0-9]* {
    if(isFunction(yytext))
	return FN;
    else {
	return ID;
    }
}

 /* All other characters are returned as-is to the parser, who can accept or
 * reject it as needed.
 */
. { return *yytext;}

%%

/* Declared in function.h, implemented here in lexer.l since this is where
 * all the yy_*() functions and types are defined.
 */
number_t parseString(const char *str)
{
    YY_BUFFER_STATE buffer = yy_scan_string(str);
    _last_parse_status_result = yyparse();
    yy_delete_buffer(buffer);
    return parse_result;
}
